<?php

namespace App\Http\Controllers;

use App\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Spatie\Permission\Models\Role;
use App\Exports\UsersExport;
use App\Imports\UsersImport;
use Maatwebsite\Excel\Facades\Excel;
use DataTables;
use Redirect;
use Response;
use Hash;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\DB;
use App\Rules\MatchOldPassword;


class UserController extends Controller
{

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('auth');
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        $users = User::latest()->get();

        if ($request->ajax()) {
            return Datatables::of($users)
                ->addIndexColumn()
                ->addColumn('actions', 'admin.pages.useractions')
                ->addColumn('roles', function ($row){
                    $roles = '';
                    foreach ($row->getRoleNames() as $roleName)
                        $roles .=  '<label class="badge badge-success">'.$roleName.'</label>&nbsp;';
                    return $roles;
                })
                ->addColumn('full_name', function ($row){
                    return $row->last_name." ".$row->middle_name." ".$row->first_name;
                })
                ->rawColumns(['actions', 'roles'])
                ->make(true);
        }
        $roles = Role::where('id','<>',1)->get();
         return view('admin.pages.manageusers', compact('users', 'roles'));
    }


    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'first_name' => 'required',
             'last_name' => 'required',
             'password' => 'same:confirm-password',
            'username' => 'required',
            'roles' => 'required'
         ]);

        if ($validator->fails()) {
            return response()->json(['type' => 'error', 'errors'=>$validator->errors()->all()]);
        }else{
            $user_id = $request->user_id;
            $user = User::updateOrCreate(['id' => $user_id],
                [
                     'first_name' => $request->first_name,
                    'last_name' => $request->last_name,
                    'middle_name' => $request->middle_name,
                    'username' => $request->username,
                    'email' => isset($request->email)?$request->email:null,
                    'password' => Hash::make($request->password)
                ]);

            $roles =$request->roles;

            if (isset($roles)) {
                $user->syncRoles($roles); // All current roles will be removed from the user and replaced by the array given
            } else {
                $user->roles()->detach(); //If no role is selected remove exisiting role associated to a user
            }
            if(empty($request->user_id))
                $msg = 'User created successfully.';
            else
                $msg = 'User data is updated successfully';
            return response()->json(['type' => 'success', 'message' => $msg]);
        }

    }


    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $where = array('id' => $id);
        $user = User::where($where)->first();
        return Response::json(['user' => $user, 'roles' => $user->roles]);

        //return Response::json($user);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $user = User::findOrFail($id); //Get user with specified id
        $user->delete();
        return response()->json(['type' => 'success', 'message' => "<div class='alert alert-success'>Successfully Deleted</div>"]);
    }

    public function export()
    {
        return Excel::download(new UsersExport, 'sampleusers.xlsx');
    }

    public function upload()
    {
        $validator = Validator::make(request()->all(), [
            'file' => 'required|mimes:csv,xlx,xls,xlsx'
        ]);

        if ($validator->fails()) {
            return response()->json(['type' => 'error', 'errors'=>$validator->errors()->all()]);
        }else{
            Excel::import(new UsersImport,request()->file('file'));
            return response()->json(['type' => 'success', 'message' => 'Users uploaded successfully.']);
        }
    }

    public function change_password(Request $request){

         $validator = Validator::make($request->all(), [
            'current_password' => ['required', new MatchOldPassword],
            'new_password' => ['required'],
            'new_confirm_password' => ['same:new_password']
        ]);

        if ($validator->fails()) {
            return response()->json(['type' => 'error', 'errors'=>$validator->errors()->all()]);
        }else{
            User::find(auth()->user()->id)->update(['password'=> Hash::make($request->new_password)]);
            return response()->json(['type' => 'success', 'message' => 'Password updated successfully.']);
        }
    }



}
